﻿namespace WarmlyLocomotove
{
    /// <summary>
    /// Параметризованный класс с параметрами типа сущности и интерфейса отрисовки дополнительных частей
    /// </summary>
    /// <typeparam name="T"></typeparam>
    /// <typeparam name="U"></typeparam>
    internal class AdvancedObjectGeneric<T, U>
        where T : EntityLocomotive
        where U : class, IDrawningAdditionalElements
    {
        /// <summary>
        /// Массив сущностей
        /// </summary>
        private T[] Entities = new T[10];
        /// <summary>
        /// Текущая выбранная сущность(индекс для заполнения)
        /// </summary>
        private int CurrentEntityIndex = 0;
        /// <summary>
        /// Массив конфигураций(набора параметров для доп. отрисовки)
        /// </summary>
        private U[] AdditionalElementsSetups = new U[10];
        /// <summary>
        /// Текущая выбранная конфигурация
        /// </summary>
        int CurrentAdditionalElementSetupIndex = 0;
        /// <summary>
        /// Добавление новой сущности в массив сущностей
        /// </summary>
        /// <param name="NewEntity">Добавляемая сущность</param>
        public void AddEntity(T NewEntity)
        {
            if (CurrentEntityIndex >= 9)
            {
                return;
            }
            Entities[CurrentEntityIndex] = NewEntity;
            CurrentEntityIndex++;
        }
        /// <summary>
        /// Добавление новой конфигурации в массив конфигураций
        /// </summary>
        /// <param name="NewAdditionalElementSetup">Добавляемая конфигурация</param>
        public void AddAdditionalElementSetup(U NewAdditionalElementSetup)
        {
            if (CurrentAdditionalElementSetupIndex >= 9)
            {
                return;
            }
            AdditionalElementsSetups[CurrentAdditionalElementSetupIndex] = NewAdditionalElementSetup;
            CurrentAdditionalElementSetupIndex++;
        }
        /// <summary>
        /// Генерация нового объекта от класса отрисовки на основе случайно выбранной сущности и набора конфигураций
        /// </summary>
        /// <returns></returns>
        public DrawningLocomotive GeneratedObject()
        {
            Random rnd = new();
            int SelectedEntityIndex = rnd.Next(0, 9);
            int SelectedAdditionalElementSetupIndex = rnd.Next(0, 9);
            DrawningLocomotive Object = null;
            if (Entities[SelectedEntityIndex] is EntityWarmlyLocomotive entitywarmlylocomotive)
            {
                Object = new DrawningWarmlyLocomotive(entitywarmlylocomotive.Speed, entitywarmlylocomotive.Weight, entitywarmlylocomotive.BodyColor, 160, 115, entitywarmlylocomotive.AdditionalColor, entitywarmlylocomotive.HasPipe, entitywarmlylocomotive.HasFuelTank);
                Object.AdditionalElements = AdditionalElementsSetups[SelectedAdditionalElementSetupIndex];
            }
            else if (Entities[SelectedEntityIndex] is EntityLocomotive entitylocomotive)
            {
                Object = new DrawningLocomotive(entitylocomotive.Speed, entitylocomotive.Weight, entitylocomotive.BodyColor);
                Object.Locomotive = Entities[SelectedEntityIndex];
                Object.AdditionalElements = AdditionalElementsSetups[SelectedAdditionalElementSetupIndex];
            }
            return Object;
        }
    }
}
