﻿namespace WarmlyLocomotove
{
    /// <summary>
    /// Класс-наследник от класса отрисовки локомотива
    /// </summary>
    internal class DrawningWarmlyLocomotive : DrawningLocomotive
    {
        /// <summary>
        /// Конструктор, передаём в protected конструктор базового класса обычные параметры и вводим новые
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Цвет кузова</param>
        /// <param name="locomotiveWidth">Ширина локомотива</param>
        /// <param name="locomotiveHeight">Высота локомотива</param>
        /// <param name="additionalColor">Дополнительный цвет</param>
        /// <param name="hasPipe">Признак наличия трубы</param>
        /// <param name="hasFuelTank">Признак наличия топливного бака</param>
        public DrawningWarmlyLocomotive(int speed, float weight, Color bodyColor, int locomotiveWidth, int locomotiveHeight, Color additionalColor, bool hasPipe, bool hasFuelTank) : base(speed, weight, bodyColor, locomotiveWidth, locomotiveHeight)
        {
            Locomotive = new EntityWarmlyLocomotive(speed, weight, bodyColor, additionalColor, hasPipe, hasFuelTank);
        }
        /// <summary>
        /// Отрисовываем базовую часть локомотива и добавляем дополнительные элементы, если они есть
        /// </summary>
        /// <param name="g"></param>
        public override void DrawTransport(Graphics g)
        {
            if (Locomotive is not EntityWarmlyLocomotive warmlyLocomotive)
            {
                return;
            }
            //Прорисовка трубы
            if (warmlyLocomotive.HasPipe)
            {
                Pen pen = new Pen(Color.Black);
                Brush brAdditionalColor = new SolidBrush(warmlyLocomotive.AdditionalColor);
                g.FillRectangle(brAdditionalColor, _startPosX + 20, _startPosY, 25, 10);
                g.DrawRectangle(pen, _startPosX + 20, _startPosY, 25, 10);
                g.FillRectangle(brAdditionalColor, _startPosX + 25, _startPosY + 10, 15, 20);
                g.DrawRectangle(pen, _startPosX + 25, _startPosY + 10, 15, 20);
            }
            _startPosY += 30;
            base.DrawTransport(g);
            _startPosY -= 30;
            //Прорисовка топливного бака
            if (warmlyLocomotive.HasFuelTank)
            {
                Pen pen = new Pen(Color.Black);
                Brush brAdditionalColor = new SolidBrush(warmlyLocomotive.AdditionalColor);
                Pen penYellow = new Pen(Color.Yellow);
                g.FillRectangle(brAdditionalColor, _startPosX + 80, _startPosY + 40, 45, 20);
                g.DrawRectangle(pen, _startPosX + 80, _startPosY + 40, 45, 20);
                for (int i = (int)_startPosX + 100; i < (int)_startPosX + 110; i++)
                {
                    g.DrawLine(penYellow, _startPosX + 105, _startPosY + 45, i, _startPosY + 55);
                }
            }
        }
    }
}
